# -*- coding: utf-8 -*-
"""
@author: P.Cizek
"""

import sys
import oscilator_network as osc
import hexapod_vrep.robot as rob
import hexapod_vrep.constants as rc
import oscilator_constants as const
import time
import math
import thread
import matplotlib.pyplot as plt

def servo_demo():
	#simple demo to control individual servos via commandline
	servo_id = 1
	done = False 
	while not done:
		mode = raw_input('a=left, d=right, n=next, p=prev, x=stop') #read command
		if mode == 'x':
			#exit
			done = True
			
		if mode == 'n':
			#select next servo
			servo_id += 1
			if servo_id == 19:
				servo_id = 1
			print('Controlling servo ' + str(servo_id)) 
			
		if mode == 'p':
			#select previous servo
			servo_id -= 1
			if servo_id == 0:
				servo_id = 18;
			print('Controlling servo ' + str(servo_id))
			
		if mode == 'a':
			#turn servo to the right
			val = robot.get_servo_position(servo_id)
			print(str(servo_id) + " = " + str(val))
			robot.set_servo_position(servo_id,val+0.1)
			
		if mode == 'd':
			#turn servo to the left
			val = robot.get_servo_position(servo_id)
			print(str(servo_id) + " = " + str(val))
			robot.set_servo_position(servo_id,val-0.1)

def locomotion_demo():
	#cpg oscillator demo
	cpg = osc.OscilatorNetwork()
	cpg.change_gait(const.TRIPOD_GAIT_WEIGHTS)

	coxa_angles= [0,0,0,0,0,0]
	cycle_length = [0,0,0,0,0,0]
	
	#steering commands
	left = 1
	right = 1
	
	for iter in range(0,num_steps):
		#get robot position and orientation in the global reference frame
		position = robot.get_robot_position()
		orientation = robot.get_robot_orientation()
		#print(position + ' ' + orientation)

		# steering - get steering speeds
		Lock.acquire()
		left = left_t
		right = right_t
		Lock.release()
		coxa_dir = [left,left,left,right,right,right]

		#next step of CPG
		cycle = cpg.oscilate_all_CPGs()
		#read the state of the network
		data = cpg.get_last_values()

		#reset coxa angles if new cycle is detected
		for i in range(0,6):
			cycle_length[i] += 1;
			if cycle[i] == True:
				coxa_angles[i]= -((cycle_length[i]-2)/2)*rc.COXA_MAX
				cycle_length[i] = 0

		#calculate individual joint angles for each of six legs
		for i in range(0,6):
			femur_val = rc.FEMUR_MAX*data[i]
			if femur_val < 0:
				coxa_angles[i] += coxa_dir[i]*rc.COXA_MAX
				femur_val *= 0.1
			else:
				coxa_angles[i] -= coxa_dir[i]*rc.COXA_MAX
			
			coxa_val = coxa_angles[i]
			tibia_val = -rc.TIBIA_MAX*data[i]
			
			#set position of each servo
			robot.set_servo_position(rc.COXA_SERVOS[i], rc.SIGN_SERVOS[i]*coxa_val+rc.COXA_OFFSETS[i])
			robot.set_servo_position(rc.FEMUR_SERVOS[i],rc.SIGN_SERVOS[i]*femur_val+rc.FEMUR_OFFSETS[i])
			robot.set_servo_position(rc.TIBIA_SERVOS[i],rc.SIGN_SERVOS[i]*tibia_val+rc.TIBIA_OFFSETS[i])
			
			#simulation step time
			time.sleep(0.001)
	
	thread.exit_thread()
	return

if __name__ == "__main__":
	#instantiate robot
	robot = rob.Robot()
	
	#simple servo control demo
	servo_demo()
	
	#simple walking demo
	Lock = thread.allocate_lock()
	l = 1
	r = 1
	left_t = l
	right_t = r
	
	#set number of gait cycles
	num_steps = 10000

	#join walking thread
	try:
		thread1 = thread.start_new_thread(locomotion_demo, ())
	except:
		print "Error: unable to start new thread"

	#online adjustent of gait parameters
	while l!=0 or r!=0:	
		Lock.acquire()
		left_t = l
		right_t = r
		Lock.release()
		l = input('Input left: ')
		r = input('Input right: ')
		l = max(-1, min(l, 1))
		r = max(-1, min(r, 1))

	#stop the simulation and disconnect from simulator
	robot.stop_simulation()
	robot.disconnect_simulator()
	sys.exit()